@@ -4,5 +4,5 @@ pub use border::*;
 mod line;
 pub use line::*;
 
-mod label;
-pub use label::*;
+mod text;
+pub use text::*;
