@@ -1,10 +1,11 @@
 use crate::view::base::*;
+use alloc::boxed::Box;
 use dep_obj::{DepObjBaseBuilder, dep_type_with_builder};
-use dep_obj::binding::{Binding, Binding1};
+use dep_obj::binding::{Binding, Binding1, Binding4};
 use dyn_context::state::{State, StateExt};
 use either::{Left, Right};
-use std::borrow::Cow;
-use std::fmt::Debug;
+use alloc::borrow::Cow;
+use core::fmt::Debug;
 use tuifw_screen_base::{Attr, Color, Point, Rect, Vector};
 use tuifw_window::RenderPort;
 
@@ -29,6 +30,10 @@ impl<'a> ViewBuilderBorderDecoratorExt for ViewBuilder<'a> {
 dep_type_with_builder! {
     #[derive(Debug)]
     pub struct BorderDecorator become decorator in View {
+        enable_l_padding: bool = true,
+        enable_t_padding: bool = true,
+        enable_r_padding: bool = true,
+        enable_b_padding: bool = true,
         tl: Cow<'static, str> = Cow::Borrowed(""),
         tr: Cow<'static, str> = Cow::Borrowed(""),
         bl: Cow<'static, str> = Cow::Borrowed(""),
@@ -37,6 +42,7 @@ dep_type_with_builder! {
         t: Cow<'static, str> = Cow::Borrowed(""),
         r: Cow<'static, str> = Cow::Borrowed(""),
         b: Cow<'static, str> = Cow::Borrowed(""),
+        fill: Cow<'static, str> = Cow::Borrowed(""),
     }
 
     type BaseBuilder<'a> = ViewBuilder<'a>;
@@ -60,6 +66,10 @@ impl Decorator for BorderDecorator {
 
 #[derive(Debug)]
 struct BorderDecoratorBindings {
+    has_l_padding: Binding<bool>,
+    has_t_padding: Binding<bool>,
+    has_r_padding: Binding<bool>,
+    has_b_padding: Binding<bool>,
     fg: Binding<Color>,
     bg: Binding<Option<Color>>,
     attr: Binding<Attr>,
@@ -71,6 +81,7 @@ struct BorderDecoratorBindings {
     t: Binding<Cow<'static, str>>,
     r: Binding<Cow<'static, str>>,
     b: Binding<Cow<'static, str>>,
+    fill: Binding<Cow<'static, str>>,
 }
 
 impl DecoratorBindings for BorderDecoratorBindings { }
@@ -79,6 +90,8 @@ impl DecoratorBindings for BorderDecoratorBindings { }
 struct BorderDecoratorBehavior;
 
 impl DecoratorBehavior for BorderDecoratorBehavior {
+    fn ty(&self) -> &'static str { "Border" }
+
     fn children_measure_size(
         &self,
         view: View,
@@ -87,20 +100,16 @@ impl DecoratorBehavior for BorderDecoratorBehavior {
     ) -> (Option<i16>, Option<i16>) {
         let tree: &ViewTree = state.get();
         let bindings = view.decorator_bindings(tree).downcast_ref::<BorderDecoratorBindings>().unwrap();
-        let tl = !bindings.tl.get_value(state).map_or(false, |x| x.is_empty());
-        let tr = !bindings.tr.get_value(state).map_or(false, |x| x.is_empty());
-        let bl = !bindings.bl.get_value(state).map_or(false, |x| x.is_empty());
-        let br = !bindings.br.get_value(state).map_or(false, |x| x.is_empty());
         let children_measure_width = if let Some(measure_width) = measure_size.0 {
-            let l = tl || bl || !bindings.l.get_value(state).map_or(false, |x| x.is_empty());
-            let r = tr || br || !bindings.r.get_value(state).map_or(false, |x| x.is_empty());
+            let l = bindings.has_l_padding.get_value(state).unwrap_or(true);
+            let r = bindings.has_r_padding.get_value(state).unwrap_or(true);
             Some((measure_width as u16).saturating_sub(if l { 1 } else { 0 }).saturating_sub(if r { 1 } else { 0 }) as i16)
         } else {
             None
         };
         let children_measure_height = if let Some(measure_height) = measure_size.1 {
-            let t = tl || tr || !bindings.t.get_value(state).map_or(false, |x| x.is_empty());
-            let b = bl || br || !bindings.b.get_value(state).map_or(false, |x| x.is_empty());
+            let t = bindings.has_t_padding.get_value(state).unwrap_or(true);
+            let b = bindings.has_b_padding.get_value(state).unwrap_or(true);
             Some((measure_height as u16).saturating_sub(if t { 1 } else { 0 }).saturating_sub(if b { 1 } else { 0 }) as i16)
         } else {
             None
@@ -111,19 +120,15 @@ impl DecoratorBehavior for BorderDecoratorBehavior {
     fn desired_size(&self, view: View, state: &mut dyn State, children_desired_size: Vector) -> Vector {
         let tree: &ViewTree = state.get();
         let bindings = view.decorator_bindings(tree).downcast_ref::<BorderDecoratorBindings>().unwrap();
-        let tl = !bindings.tl.get_value(state).map_or(false, |x| x.is_empty());
-        let tr = !bindings.tr.get_value(state).map_or(false, |x| x.is_empty());
-        let bl = !bindings.bl.get_value(state).map_or(false, |x| x.is_empty());
-        let br = !bindings.br.get_value(state).map_or(false, |x| x.is_empty());
-        let l = tl || bl || !bindings.l.get_value(state).map_or(false, |x| x.is_empty());
-        let r = tr || br || !bindings.r.get_value(state).map_or(false, |x| x.is_empty());
+        let l = bindings.has_l_padding.get_value(state).unwrap_or(true);
+        let r = bindings.has_r_padding.get_value(state).unwrap_or(true);
         let desired_width = (children_desired_size.x as u16)
             .saturating_add(if l { 1 } else { 0 })
             .saturating_add(if r { 1 } else { 0 })
             as i16
         ;
-        let t = tl || tr || !bindings.t.get_value(state).map_or(false, |x| x.is_empty());
-        let b = bl || br || !bindings.b.get_value(state).map_or(false, |x| x.is_empty());
+        let t = bindings.has_t_padding.get_value(state).unwrap_or(true);
+        let b = bindings.has_b_padding.get_value(state).unwrap_or(true);
         let desired_height = (children_desired_size.y as u16)
             .saturating_add(if t { 1 } else { 0 })
             .saturating_add(if b { 1 } else { 0 })
@@ -135,18 +140,14 @@ impl DecoratorBehavior for BorderDecoratorBehavior {
     fn children_arrange_bounds(&self, view: View, state: &mut dyn State, arrange_size: Vector) -> Rect {
         let tree: &ViewTree = state.get();
         let bindings = view.decorator_bindings(tree).downcast_ref::<BorderDecoratorBindings>().unwrap();
-        let tl = !bindings.tl.get_value(state).map_or(false, |x| x.is_empty());
-        let tr = !bindings.tr.get_value(state).map_or(false, |x| x.is_empty());
-        let bl = !bindings.bl.get_value(state).map_or(false, |x| x.is_empty());
-        let br = !bindings.br.get_value(state).map_or(false, |x| x.is_empty());
-        let l = tl || bl || !bindings.l.get_value(state).map_or(false, |x| x.is_empty());
-        let t = tl || tr || !bindings.t.get_value(state).map_or(false, |x| x.is_empty());
+        let l = bindings.has_l_padding.get_value(state).unwrap_or(true);
+        let t = bindings.has_t_padding.get_value(state).unwrap_or(true);
         let tl_offset = Point {
             x: if l { 1 } else { 0 },
             y: if t { 1 } else { 0 },
         };
-        let r = tr || br || !bindings.r.get_value(state).map_or(false, |x| x.is_empty());
-        let b = bl || br || !bindings.b.get_value(state).map_or(false, |x| x.is_empty());
+        let r = bindings.has_r_padding.get_value(state).unwrap_or(true);
+        let b = bindings.has_b_padding.get_value(state).unwrap_or(true);
         let br_offset = Vector {
             x: if r { -1 } else { 0 },
             y: if b { -1 } else { 0 },
@@ -158,18 +159,14 @@ impl DecoratorBehavior for BorderDecoratorBehavior {
     fn render_bounds(&self, view: View, state: &mut dyn State, children_render_bounds: Rect) -> Rect {
         let tree: &ViewTree = state.get();
         let bindings = view.decorator_bindings(tree).downcast_ref::<BorderDecoratorBindings>().unwrap();
-        let tl = !bindings.tl.get_value(state).map_or(false, |x| x.is_empty());
-        let tr = !bindings.tr.get_value(state).map_or(false, |x| x.is_empty());
-        let bl = !bindings.bl.get_value(state).map_or(false, |x| x.is_empty());
-        let br = !bindings.br.get_value(state).map_or(false, |x| x.is_empty());
-        let l = tl || bl || !bindings.l.get_value(state).map_or(false, |x| x.is_empty());
-        let t = tl || tr || !bindings.t.get_value(state).map_or(false, |x| x.is_empty());
+        let l = bindings.has_l_padding.get_value(state).unwrap_or(true);
+        let t = bindings.has_t_padding.get_value(state).unwrap_or(true);
         let tl_offset = Vector {
             x: if l { -1 } else { 0 },
             y: if t { -1 } else { 0 },
         };
-        let r = tr || br || !bindings.r.get_value(state).map_or(false, |x| x.is_empty());
-        let b = bl || br || !bindings.b.get_value(state).map_or(false, |x| x.is_empty());
+        let r = bindings.has_r_padding.get_value(state).unwrap_or(true);
+        let b = bindings.has_b_padding.get_value(state).unwrap_or(true);
         let br_offset = Vector {
             x: if r { 1 } else { 0 },
             y: if b { 1 } else { 0 },
@@ -204,6 +201,7 @@ impl DecoratorBehavior for BorderDecoratorBehavior {
         let r: &str = &bindings.r.get_value(state).unwrap_or(Cow::Borrowed(""));
         let t: &str = &bindings.t.get_value(state).unwrap_or(Cow::Borrowed(""));
         let b: &str = &bindings.b.get_value(state).unwrap_or(Cow::Borrowed(""));
+        let fill: &str = &bindings.fill.get_value(state).unwrap_or(Cow::Borrowed(""));
         let l = if !l.is_empty() { l } else if !tl.is_empty() || !bl.is_empty() { " " } else { "" };
         let t = if !t.is_empty() { t } else if !tl.is_empty() || !tr.is_empty() { " " } else { "" };
         let r = if !r.is_empty() { r } else if !tr.is_empty() || !br.is_empty() { " " } else { "" };
@@ -211,6 +209,9 @@ impl DecoratorBehavior for BorderDecoratorBehavior {
         let fg = bindings.fg.get_value(state).unwrap_or(Color::White);
         let bg = bindings.bg.get_value(state).unwrap_or_default();
         let attr = bindings.attr.get_value(state).unwrap_or_default();
+        if !fill.is_empty() {
+            port.fill(|port, p| port.out(p, fg, bg, attr, fill));
+        }
         if !l.is_empty() {
             for y in 0 .. size.y as u16 {
                 port.out(Point { x: 0, y: y as i16 }, fg, bg, attr, l);
@@ -247,6 +248,38 @@ impl DecoratorBehavior for BorderDecoratorBehavior {
     }
 
     fn init_bindings(&self, view: View, state: &mut dyn State) -> Box<dyn DecoratorBindings> {
+        let has_l_padding = Binding4::new(state, (), |(),
+            l: Cow<'static, str>,
+            tl: Cow<'static, str>,
+            bl: Cow<'static, str>,
+            enable: bool
+        |
+            Some(enable && (!l.is_empty() || !tl.is_empty() || !bl.is_empty()))
+        );
+        let has_t_padding = Binding4::new(state, (), |(),
+            t: Cow<'static, str>,
+            tl: Cow<'static, str>,
+            tr: Cow<'static, str>,
+            enable: bool
+        |
+            Some(enable && (!t.is_empty() || !tl.is_empty() || !tr.is_empty()))
+        );
+        let has_r_padding = Binding4::new(state, (), |(),
+            r: Cow<'static, str>,
+            tr: Cow<'static, str>,
+            br: Cow<'static, str>,
+            enable: bool
+        |
+            Some(enable && (!r.is_empty() || !tr.is_empty() || !br.is_empty()))
+        );
+        let has_b_padding = Binding4::new(state, (), |(),
+            b: Cow<'static, str>,
+            bl: Cow<'static, str>,
+            br: Cow<'static, str>,
+            enable: bool
+        |
+            Some(enable && (!b.is_empty() || !bl.is_empty() || !br.is_empty()))
+        );
         let bg = Binding1::new(state, (), |(), bg| Some(bg));
         let fg = Binding1::new(state, (), |(), fg| Some(fg));
         let attr = Binding1::new(state, (), |(), attr| Some(attr));
@@ -258,6 +291,91 @@ impl DecoratorBehavior for BorderDecoratorBehavior {
         let t = Binding1::new(state, (), |(), t| Some(t));
         let r = Binding1::new(state, (), |(), r| Some(r));
         let b = Binding1::new(state, (), |(), b| Some(b));
+        let fill = Binding1::new(state, (), |(), fill| Some(fill));
+        has_l_padding.set_target_fn(state, view, |state, view, _| view.invalidate_measure(state));
+        has_t_padding.set_target_fn(state, view, |state, view, _| view.invalidate_measure(state));
+        has_r_padding.set_target_fn(state, view, |state, view, _| view.invalidate_measure(state));
+        has_b_padding.set_target_fn(state, view, |state, view, _| view.invalidate_measure(state));
+        bg.set_target_fn(state, view, |state, view, _| view.invalidate_render(state));
+        fg.set_target_fn(state, view, |state, view, _| view.invalidate_render(state));
+        attr.set_target_fn(state, view, |state, view, _| view.invalidate_render(state));
+        tl.set_target_fn(state, view, |state, view, _| view.invalidate_rect(state, Rect {
+            tl: Point { x: 0, y: 0 },
+            size: Vector { x: 1, y: 1 }
+        }));
+        tr.set_target_fn(state, view, |state, view, _| {
+            let tree: &ViewTree = state.get();
+            let size = view.render_bounds(tree).size;
+            view.invalidate_rect(state, Rect {
+                tl: Point { x: size.x.wrapping_sub(1), y: 0 },
+                size: Vector { x: 1, y: 1 }
+            });
+        });
+        bl.set_target_fn(state, view, |state, view, _| {
+            let tree: &ViewTree = state.get();
+            let size = view.render_bounds(tree).size;
+            view.invalidate_rect(state, Rect {
+                tl: Point { x: 0, y: size.y.wrapping_sub(1) },
+                size: Vector { x: 1, y: 1 }
+            });
+        });
+        br.set_target_fn(state, view, |state, view, _| {
+            let tree: &ViewTree = state.get();
+            let size = view.render_bounds(tree).size;
+            view.invalidate_rect(state, Rect {
+                tl: Point { x: size.x.wrapping_sub(1), y: size.y.wrapping_sub(1) },
+                size: Vector { x: 1, y: 1 }
+            });
+        });
+        l.set_target_fn(state, view, |state, view, _| {
+            let tree: &ViewTree = state.get();
+            let size = view.render_bounds(tree).size;
+            view.invalidate_rect(state, Rect {
+                tl: Point { x: 0, y: 0 },
+                size: Vector { x: 1, y: size.y }
+            });
+        });
+        t.set_target_fn(state, view, |state, view, _| {
+            let tree: &ViewTree = state.get();
+            let size = view.render_bounds(tree).size;
+            view.invalidate_rect(state, Rect {
+                tl: Point { x: 0, y: 0 },
+                size: Vector { x: size.x, y: 1 }
+            });
+        });
+        r.set_target_fn(state, view, |state, view, _| {
+            let tree: &ViewTree = state.get();
+            let size = view.render_bounds(tree).size;
+            view.invalidate_rect(state, Rect {
+                tl: Point { x: size.x.wrapping_sub(1), y: 0 },
+                size: Vector { x: 1, y: size.y }
+            });
+        });
+        b.set_target_fn(state, view, |state, view, _| {
+            let tree: &ViewTree = state.get();
+            let size = view.render_bounds(tree).size;
+            view.invalidate_rect(state, Rect {
+                tl: Point { x: 0, y: size.y.wrapping_sub(1) },
+                size: Vector { x: size.x, y: 1 }
+            });
+        });
+        fill.set_target_fn(state, view, |state, view, _| view.invalidate_render(state));
+        has_l_padding.set_source_1(state, &mut BorderDecorator::L.value_source(view.decorator()));
+        has_l_padding.set_source_2(state, &mut BorderDecorator::TL.value_source(view.decorator()));
+        has_l_padding.set_source_3(state, &mut BorderDecorator::BL.value_source(view.decorator()));
+        has_l_padding.set_source_4(state, &mut BorderDecorator::ENABLE_L_PADDING.value_source(view.decorator()));
+        has_t_padding.set_source_1(state, &mut BorderDecorator::T.value_source(view.decorator()));
+        has_t_padding.set_source_2(state, &mut BorderDecorator::TL.value_source(view.decorator()));
+        has_t_padding.set_source_3(state, &mut BorderDecorator::TR.value_source(view.decorator()));
+        has_t_padding.set_source_4(state, &mut BorderDecorator::ENABLE_T_PADDING.value_source(view.decorator()));
+        has_r_padding.set_source_1(state, &mut BorderDecorator::R.value_source(view.decorator()));
+        has_r_padding.set_source_2(state, &mut BorderDecorator::TR.value_source(view.decorator()));
+        has_r_padding.set_source_3(state, &mut BorderDecorator::BR.value_source(view.decorator()));
+        has_r_padding.set_source_4(state, &mut BorderDecorator::ENABLE_R_PADDING.value_source(view.decorator()));
+        has_b_padding.set_source_1(state, &mut BorderDecorator::B.value_source(view.decorator()));
+        has_b_padding.set_source_2(state, &mut BorderDecorator::BL.value_source(view.decorator()));
+        has_b_padding.set_source_3(state, &mut BorderDecorator::BR.value_source(view.decorator()));
+        has_b_padding.set_source_4(state, &mut BorderDecorator::ENABLE_B_PADDING.value_source(view.decorator()));
         bg.set_source_1(state, &mut ViewBase::BG.value_source(view.base()));
         fg.set_source_1(state, &mut ViewBase::FG.value_source(view.base()));
         attr.set_source_1(state, &mut ViewBase::ATTR.value_source(view.base()));
@@ -269,82 +387,12 @@ impl DecoratorBehavior for BorderDecoratorBehavior {
         t.set_source_1(state, &mut BorderDecorator::T.value_source(view.decorator()));
         r.set_source_1(state, &mut BorderDecorator::R.value_source(view.decorator()));
         b.set_source_1(state, &mut BorderDecorator::B.value_source(view.decorator()));
-        bg.set_target_fn(state, view, |state, view, _| {
-            let tree: &mut ViewTree = state.get_mut();
-            view.invalidate_render(tree).expect("invalidate_render failed");
-        });
-        fg.set_target_fn(state, view, |state, view, _| {
-            let tree: &mut ViewTree = state.get_mut();
-            view.invalidate_render(tree).expect("invalidate_render failed");
-        });
-        attr.set_target_fn(state, view, |state, view, _| {
-            let tree: &mut ViewTree = state.get_mut();
-            view.invalidate_render(tree).expect("invalidate_render failed");
-        });
-        tl.set_target_fn(state, view, |state, view, _| {
-            let tree: &mut ViewTree = state.get_mut();
-            view.invalidate_rect(tree, Rect {
-                tl: Point { x: 0, y: 0 },
-                size: Vector { x: 1, y: 1 }
-            }).unwrap();
-        });
-        tr.set_target_fn(state, view, |state, view, _| {
-            let tree: &mut ViewTree = state.get_mut();
-            let size = view.render_bounds(tree).size;
-            view.invalidate_rect(tree, Rect {
-                tl: Point { x: size.x.wrapping_sub(1), y: 0 },
-                size: Vector { x: 1, y: 1 }
-            }).unwrap();
-        });
-        bl.set_target_fn(state, view, |state, view, _| {
-            let tree: &mut ViewTree = state.get_mut();
-            let size = view.render_bounds(tree).size;
-            view.invalidate_rect(tree, Rect {
-                tl: Point { x: 0, y: size.y.wrapping_sub(1) },
-                size: Vector { x: 1, y: 1 }
-            }).unwrap();
-        });
-        br.set_target_fn(state, view, |state, view, _| {
-            let tree: &mut ViewTree = state.get_mut();
-            let size = view.render_bounds(tree).size;
-            view.invalidate_rect(tree, Rect {
-                tl: Point { x: size.x.wrapping_sub(1), y: size.y.wrapping_sub(1) },
-                size: Vector { x: 1, y: 1 }
-            }).unwrap();
-        });
-        l.set_target_fn(state, view, |state, view, _| {
-            let tree: &mut ViewTree = state.get_mut();
-            let size = view.render_bounds(tree).size;
-            view.invalidate_rect(tree, Rect {
-                tl: Point { x: 0, y: 0 },
-                size: Vector { x: 1, y: size.y }
-            }).unwrap();
-        });
-        t.set_target_fn(state, view, |state, view, _| {
-            let tree: &mut ViewTree = state.get_mut();
-            let size = view.render_bounds(tree).size;
-            view.invalidate_rect(tree, Rect {
-                tl: Point { x: 0, y: 0 },
-                size: Vector { x: size.x, y: 1 }
-            }).unwrap();
-        });
-        r.set_target_fn(state, view, |state, view, _| {
-            let tree: &mut ViewTree = state.get_mut();
-            let size = view.render_bounds(tree).size;
-            view.invalidate_rect(tree, Rect {
-                tl: Point { x: size.x.wrapping_sub(1), y: 0 },
-                size: Vector { x: 1, y: size.y }
-            }).unwrap();
-        });
-        b.set_target_fn(state, view, |state, view, _| {
-            let tree: &mut ViewTree = state.get_mut();
-            let size = view.render_bounds(tree).size;
-            view.invalidate_rect(tree, Rect {
-                tl: Point { x: 0, y: size.y.wrapping_sub(1) },
-                size: Vector { x: size.x, y: 1 }
-            }).unwrap();
-        });
+        fill.set_source_1(state, &mut BorderDecorator::FILL.value_source(view.decorator()));
         Box::new(BorderDecoratorBindings {
+            has_l_padding: has_l_padding.into(),
+            has_t_padding: has_t_padding.into(),
+            has_r_padding: has_r_padding.into(),
+            has_b_padding: has_b_padding.into(),
             bg: bg.into(),
             fg: fg.into(),
             attr: attr.into(),
@@ -356,11 +404,16 @@ impl DecoratorBehavior for BorderDecoratorBehavior {
             t: t.into(),
             r: r.into(),
             b: b.into(),
+            fill: fill.into(),
         })
     }
 
     fn drop_bindings(&self, _view: View, state: &mut dyn State, bindings: Box<dyn DecoratorBindings>) {
         let bindings = bindings.downcast::<BorderDecoratorBindings>().unwrap();
+        bindings.has_l_padding.drop_binding(state);
+        bindings.has_t_padding.drop_binding(state);
+        bindings.has_r_padding.drop_binding(state);
+        bindings.has_b_padding.drop_binding(state);
         bindings.bg.drop_binding(state);
         bindings.fg.drop_binding(state);
         bindings.attr.drop_binding(state);
@@ -372,5 +425,6 @@ impl DecoratorBehavior for BorderDecoratorBehavior {
         bindings.t.drop_binding(state);
         bindings.r.drop_binding(state);
         bindings.b.drop_binding(state);
+        bindings.fill.drop_binding(state);
     }
 }
